/* Copyright (C) 2004 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

/**
 * @file myx_gc_style.cpp 
 * @brief Implementation of the base style class.
 * 
 */

#include "myx_gc.h"
#include "myx_gc_style.h"
#include "myx_gc_figure.h"

//----------------- CGCStyle -------------------------------------------------------------------------------------------

CGCStyle::CGCStyle(wstring Name)
{
  FName = Name;
  FDisplayList = 0;
  FDestroying = false;
}

//----------------------------------------------------------------------------------------------------------------------

CGCStyle::~CGCStyle(void)
{
  FDestroying = true;
  glDeleteLists(FDisplayList, 1);
  for (CElementSet::iterator Iterator = FListeners.begin(); Iterator != FListeners.end(); ++Iterator)
    (*Iterator)->UseStyle(NULL);
}

//----------------------------------------------------------------------------------------------------------------------

void CGCStyle::AddListener(CFigureElement* Element)
{
  if (!FDestroying)
    FListeners.insert(Element);
}

//----------------------------------------------------------------------------------------------------------------------

TBoundingBox* CGCStyle::BoundingBox(void)
{
  return &FBoundingBox;
}

//----------------------------------------------------------------------------------------------------------------------

GLuint CGCStyle::DisplayList(void)
{
  return FDisplayList;
}

//----------------------------------------------------------------------------------------------------------------------

/**
 * Retrieves the value of the property given by path. The path syntax is must be something like (here expressed as regex)
 * (container)*(property), where container is a slash and the name of a container class (e.g. layers, figures) and
 * property is the name of a simple property of that container.
 *
 * @param Path The path of the property to return.
 * @return A description of the property value and, if the property is simple, the actual value.
 */
TGCVariant CGCStyle::Property(string Path)
{
  TGCVariant Result;

  // Extract top container, adjust Path and iterate down if necessary.
  switch (GetTopContainer(Path))
  {
    case GC_CONTAINER_ROOT:
      {
        switch (GetPropertyID(Path))
        {
          case GC_PROPERTY_NAME:
            {
              Result.Type = GC_VAR_STRING;
              Result.S = Utf16ToUtf8(FName);
              break;
            };
        };
        break;
      };
  };

  return Result;
}

//----------------------------------------------------------------------------------------------------------------------

/**
 * Sets the value of the given property, which must be a simple property.
 *
 * @param The path of the property.
 * @param Value The new value of the property. Automatic conversion is performed where possible.
 */
void CGCStyle::Property(string Path, const TGCVariant& Value)
{
  // There are currently no properties that could be changed. The name is a unique identifier and must not be changed.
}

//----------------------------------------------------------------------------------------------------------------------

void CGCStyle::RemoveListener(CFigureElement* Element)
{
  if (!FDestroying)
    FListeners.erase(Element);
}

//----------------------------------------------------------------------------------------------------------------------

