#import "MQResultSetView.h"
#import "MQResultSetDataSource.h"


typedef struct {
  int error;
  NSString *message;
} MessageItem;


@interface MessageDS : NSObject
{
  MessageItem *items;
  int itemCount;
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView;
- (id)tableView:(NSTableView *)aTableView 
objectValueForTableColumn:(NSTableColumn *)aTableColumn
			row:(int)rowIndex;

- (void)addError:(int)error
		 message:(NSString*)message;

@end

@implementation MessageDS

- (id)init
{
  self= [super init];
  if (self)
  {
	items= NULL;
	itemCount= 0;
  }
  return self;
}


- (void)dealloc
{
  int i;  
  for (i= 0; i < itemCount; i++)
  {
	[items[i].message release];
  }
  g_free(items);
  [super dealloc];
}


- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
  return itemCount;
}


- (id)tableView:(NSTableView *)aTableView 
objectValueForTableColumn:(NSTableColumn *)aTableColumn
			row:(int)rowIndex
{
  if ([[aTableColumn identifier] isEqualTo:@"error"])
	return [NSNumber numberWithInt:items[rowIndex].error];
  else
	return items[rowIndex].message;
}


- (void)addError:(int)error
		 message:(NSString*)message
{
  itemCount++;
  items= g_realloc(items, sizeof(MessageItem)*itemCount);
  items[itemCount-1].error= error;
  items[itemCount-1].message= [message retain];
}

@end


//=============================================================================


@implementation MQResultSetView

- (void)updateButtonBar
{
  if (_dataSource)
  {
	if ([_dataSource editing])
	{
	  [[view viewWithTag:50] setEnabled:NO];
	  [[view viewWithTag:51] setEnabled:YES];
	  [[view viewWithTag:52] setEnabled:YES];
	}
	else
	{
	  if ([_dataSource isEditable])
		[[view viewWithTag:50] setEnabled:YES];
	  else
		[[view viewWithTag:50] setEnabled:NO];
	  [[view viewWithTag:51] setEnabled:NO];
	  [[view viewWithTag:52] setEnabled:NO];
	}
	[[view viewWithTag:53] setEnabled:YES];
	[[view viewWithTag:54] setEnabled:YES];
	[[view viewWithTag:55] setEnabled:NO];
  }
  else
  {
	[[view viewWithTag:50] setEnabled:NO];
	[[view viewWithTag:51] setEnabled:NO];
	[[view viewWithTag:52] setEnabled:NO];
	
	[[view viewWithTag:53] setEnabled:NO];
	[[view viewWithTag:54] setEnabled:NO];
	[[view viewWithTag:55] setEnabled:NO];	
  }
}


- (IBAction)addRow:(id)sender
{
  [_dataSource addRow];
  [table reloadData];
}

- (IBAction)deleteRow:(id)sender
{
  [_dataSource deleteRow:[table selectedRow]];
  [table reloadData];
}

- (IBAction)copyField:(id)sender
{
}

- (IBAction)saveField:(id)sender
{
}

- (IBAction)loadField:(id)sender
{
}

- (IBAction)clearField:(id)sender
{
}

- (IBAction)editField:(id)sender
{
}

- (IBAction)viewField:(id)sender
{
}


- (IBAction)editRS:(id)sender
{
  [_dataSource setEditing:YES];
  [self updateButtonBar];
  [statusLabel setStringValue:@"Editing enabled."];
}

- (IBAction)commitRS:(id)sender
{
  [_dataSource setEditing:NO];
  [self updateButtonBar];
  
  [statusLabel setStringValue:@"Saving changes..."];
  
  [_dataSource prepareForSave];
  
  [self performApply];
}

- (IBAction)discardRS:(id)sender
{
  [_dataSource setEditing:NO];
  [_dataSource discardChanges];
  [self updateButtonBar];
  [table reloadData];
  [statusLabel setStringValue:@"Discarded changes."];
}

- (IBAction)goFirst:(id)sender
{
  if ([_dataSource numberOfRowsInTableView:table] > 0)
	[table selectRow:0 byExtendingSelection:NO];
}

- (IBAction)goLast:(id)sender
{
  if ([_dataSource numberOfRowsInTableView:table] > 0)
	[table selectRow:[_dataSource numberOfRowsInTableView:table]-1
byExtendingSelection:NO];
}

- (IBAction)searchRS:(id)sender
{
}

- (id)initWithConnectionTo:(MYX_USER_CONNECTION*)info
{
  self= [super init];
  if (self)
  {
	if (![NSBundle loadNibNamed:@"ResultSet" owner:self])
	{
	  NSLog(@"Could not load nib ResultSet.nib");
	  [self release];
	  return nil;
	}

	_mysql= myx_mysql_init();
	if (!_mysql)
	{
	  NSLog(@"could not create MySQL connection");
	  [self release];
	  return nil;
	}
	if (myx_connect_to_instance(info, _mysql) < 0)
	{
	  //XXX show error message with reason etc
	  [self release];
	  return nil;
	}
	
	_messageDS= [[MessageDS alloc] init];
	[messageTable setDataSource:_messageDS];
	
	[self updateButtonBar];
		
	_statusColors[MQUnchanged] = nil;
	_statusColors[MQChanged] = [[NSColor colorWithDeviceRed:0.8 green:0.8 blue:1.0 alpha:1.0] retain];
	_statusColors[MQAdded] = [[NSColor colorWithDeviceRed:0.8 green:1.0 blue:0.8 alpha:1.0] retain];
	_statusColors[MQDeleted] = [[NSColor colorWithDeviceRed:1.0 green:0.8 blue:0.8 alpha:1.0] retain];
  }
  return self;
}

- (void)setDefaultSchema:(NSString*)schema
{
  myx_use_schema(_mysql, [schema UTF8String]);
}

- (void)dealloc
{
  int i;
  for (i= 0; i < 4; i++)
	[_statusColors[i] release];

  [_query release];
  if (_mysql)
	myx_mysql_close(_mysql);
  [_dataSource release];
  
  [_messageDS release];
  
  [super dealloc];
}

- (id)view
{
  return view;
}


- (void)setDelegate:(id)delegate
{
  _delegate= delegate;
}


- (void)queryProgress:(NSDictionary*)info
{
  
}

static int fetchProgress(unsigned long current_row_count, 
						 unsigned long previous_row_count, 
						 MYX_RESULTSET *result_set, void *user_data)
{
  NSDictionary *dict;
  
  dict= [NSDictionary dictionaryWithObjectsAndKeys:
	[NSNumber numberWithLong:current_row_count], @"current",
	[NSNumber numberWithLong:previous_row_count], @"previous",
	[NSValue valueWithPointer:result_set], @"resultset",
	nil];
  
  [(id)user_data performSelectorOnMainThread:@selector(queryProgress:)
								  withObject:dict
							   waitUntilDone:NO];
  
  return 0;
}


- (void)displayResults:(NSDictionary*)info
{
  MYX_RESULTSET *rs= [[info valueForKey:@"resultset"] pointerValue];
  unsigned int i;
  NSArray *columns= [table tableColumns];
  
  _busy= NO;

  if (!rs)
  {
	[statusLabel setStringValue:[NSString stringWithFormat:@"Error executing query."]];
	char *text;
	NSSize size;

	text= myx_mysql_error(_mysql);
	
	[_messageDS addError:myx_mysql_errno(_mysql)
				 message:[[NSString alloc] initWithBytesNoCopy:text
														length:strlen(text)
													  encoding:NSUTF8StringEncoding
												  freeWhenDone:YES]];
	[messageTable reloadData];
	size= [messageTable frame].size;
	[messageTable setFrameSize: NSMakeSize(size.width, 100)];

	[_delegate resultSetViewFinishedQuery:self];
	return;
  }
  [_dataSource release];
  
  _dataSource= [[MQResultSetDataSource alloc] initWithResultSet:rs];
  for (i= 0; i < [columns count]; i++)
	[table removeTableColumn:[columns objectAtIndex:0]];

  for (i= 0; i < rs->columns_num; i++)
  {
	NSTableColumn *column= [[NSTableColumn alloc] initWithIdentifier:[NSNumber numberWithInt:i]];
	NSString *title= [NSString stringWithUTF8String:rs->columns[i].name];
	float width= 0.0;
	float twidth;

	switch (rs->columns[i].column_type)
	{
	  case MYX_RSCT_INTEGER:
		width= [[[column dataCell] font] widthOfString:@"0000000000"];
		break;
	  case MYX_RSCT_FLOAT:
		width= [[[column dataCell] font] widthOfString:@"0000000000"];
		break;
	  case MYX_RSCT_STRING:
		width= [[[column dataCell] font] widthOfString:@"XYZW"] * rs->columns[i].type_size / 3;
		if (width < 50.0)
		  width= 50.0;
		break;
	  case MYX_RSCT_DATE:
		width= [[[column dataCell] font] widthOfString:@"00/00/0000 "];
		break;
	  case MYX_RSCT_TIME:
		width= [[[column dataCell] font] widthOfString:@"00:00:00.0000"];
		break;
	  case MYX_RSCT_DATETIME:
		width= [[[column dataCell] font] widthOfString:@"00/00/0000  00:00:00.0000"];
		break;
	  case MYX_RSCT_BLOB:
		width= 50;
		break;
	  case MYX_RSCT_TEXT:
		width= 250;
		break;
	  case MYX_RSCT_ENUM:
		width= 100;
		break;
	  case MYX_RSCT_SET:
		width= 250;
		break;
	}
	
	twidth= [[[column headerCell] font] widthOfString:title];
	width= MIN(MAX(width, twidth), 250);

	if (rs->columns[i].table_column->is_pk)
	  [[column headerCell] setStringValue:[NSString stringWithFormat:@"%@ %@",
		[NSString stringWithUTF8String:"\xef\x83\x91"], title]];
	else
	  [[column headerCell] setStringValue:title];
	[column setWidth:width];
	
	[[column dataCell] setDrawsBackground:NO];

	[table addTableColumn:column];
  }

  [table setDataSource:_dataSource];
  [table reloadData];
  
  [statusLabel setStringValue:[NSString stringWithFormat:@"%i rows fetched.", 
	rs->rows_num]];
  
  [_delegate resultSetViewFinishedQuery:self];
  
  [self updateButtonBar];
}

- (void)queryFinished:(NSDictionary*)info
{
  // we're being called from a performSelectorOnMainThread, so
  // set up to display the results later, when we're already running normally
  [[NSRunLoop currentRunLoop] performSelector:@selector(displayResults:)
									   target:self
									 argument:info
										order:100
										modes:[NSArray arrayWithObject:NSDefaultRunLoopMode]];
}

- (void)doQuery:(NSDictionary*)args
{
  MYSQL *mysql= [[args objectForKey:@"mysql"] pointerValue];
  NSString *query= [args objectForKey:@"query"];
  MYX_STRINGLIST *params= [[args objectForKey:@"params"] pointerValue];
  MYX_LIB_ERROR error_code;
  MYX_RESULTSET *result;
  NSDictionary *info;
  
  result= myx_query_execute(mysql, [query UTF8String],
							TRUE,
							params, &error_code, self,
							fetchProgress,
							NULL,
							NULL);

  info= [NSDictionary dictionaryWithObjectsAndKeys:
	[NSValue valueWithPointer:result], @"resultset",
	[NSNumber numberWithInt:error_code], @"error", nil];

  [self performSelectorOnMainThread:@selector(queryFinished:)
						 withObject:info
					  waitUntilDone:YES];
}


- (void)performQuery:(NSString*)query
{
  [self setQuery:query];
  
  [statusLabel setStringValue:@"Performing query..."];
  
  [_delegate resultSetView:self willStartQuery:query];
  
  [NSApplication detachDrawingThread:@selector(doQuery:)
							toTarget:self
						  withObject:[NSDictionary dictionaryWithObjectsAndKeys:
							[NSValue valueWithPointer:_mysql], @"mysql",
							query, @"query",
							nil]];
  _busy= YES;
}


- (void)applyFinished:(id)errorsVal
{
  MYX_RS_ACTION_ERRORS *errors= [errorsVal pointerValue];

  if (errors)
  {
	unsigned int i;
	//XXX display errors
	for (i= 0; i < errors->errors_num; i++)
	{
	  
	}

	myx_query_free_action_errors(errors);
  }
  else
  {
	[statusLabel setStringValue:@"Applied actions."];
  }
  [_dataSource commitActions];
  [table reloadData];

  _busy= NO;
}

- (void)doApply:(NSDictionary*)args
{
  MYSQL *mysql= [[args objectForKey:@"mysql"] pointerValue];
  MYX_RESULTSET *rset= [[args objectForKey:@"resultset"] pointerValue];
  MYX_RS_ACTION_ERRORS *errors;
  
  errors= myx_query_apply_actions(rset);
  
  [self performSelectorOnMainThread:@selector(applyFinished:)
						 withObject:[NSValue valueWithPointer:errors]
					  waitUntilDone:YES];
}


- (void)performApply
{
  [statusLabel setStringValue:@"Applying actions..."];

//  [_delegate resultSetView:self willStartQuery:query];
  
  [NSApplication detachDrawingThread:@selector(doApply:)
							toTarget:self
						  withObject:[NSDictionary dictionaryWithObjectsAndKeys:
							[NSValue valueWithPointer:_mysql], @"mysql",
							[NSValue valueWithPointer:[_dataSource resultset]], @"resultset",
							nil]];
  _busy= YES;
}


- (BOOL)isBusy
{
  return _busy;
}

- (void)setQuery:(NSString*)query
{
  if (_query != query)
  {
	[_query release];
	_query= [query retain];
  }
}

- (NSString*)query
{
  return _query;
}


- (BOOL)tableView:(NSTableView *)aTableView 
shouldEditTableColumn:(NSTableColumn *)aTableColumn 
			  row:(int)rowIndex
{
  int column= [[aTableColumn identifier] intValue];
  if ([_dataSource editing] && [_dataSource statusOfRow:rowIndex
												 column:column] != MQDeleted)
	return YES;
  else
	return NO;
}

- (int)numberOfItemsInMenu:(NSMenu *)menu
{
  return [menu numberOfItems];
}

- (BOOL)menu:(NSMenu *)menu
  updateItem:(NSMenuItem *)item
	 atIndex:(int)index
shouldCancel:(BOOL)shouldCancel
{
  BOOL flag= NO;
  BOOL hasSelection= [table selectedRow]>=0;

  switch (index)
  {
	case 0: //add
	  flag= [_dataSource editing];
	  break;
	case 1: //delete
	  flag= [_dataSource editing] && hasSelection;
	  break;
	case 3: //load
	  flag= [_dataSource editing] && hasSelection;
	  break;
	case 4: //save
	  flag= hasSelection;
	  break;
	case 5: //copy
	  flag= hasSelection;
	  break;
	case 6: //clear
	  flag= [_dataSource editing] && hasSelection;
	  break;	  
	case 8: //view
	  flag= hasSelection;
	  break;
	case 9: //edit
	  flag= [_dataSource editing] && hasSelection;
	  break;
  }
  [item setEnabled:flag];

  return YES;
}


- (void)tableView:(NSTableView *)aTableView 
  willDisplayCell:(id)aCell
   forTableColumn:(NSTableColumn *)aTableColumn
			  row:(int)rowIndex
{
  if ([_dataSource editing])
  {
	int column= [[aTableColumn identifier] intValue];
	
	switch ([_dataSource statusOfRow:rowIndex column:column])
	{
	  case MQUnchanged:
		break;
	  case MQChanged:
		[aCell setBackgroundColor:_statusColors[MQChanged]];
		[aCell setDrawsBackground:YES];
		return;
	  case MQAdded:
		[aCell setBackgroundColor:_statusColors[MQAdded]];
		[aCell setDrawsBackground:YES];
		return;		
	  case MQDeleted:
		[aCell setBackgroundColor:_statusColors[MQDeleted]];
		[aCell setDrawsBackground:YES];
		return;		
	}
  }
  [aCell setDrawsBackground:NO];
}

@end

