//
//  MQueryBrowser.h
//  MySQL QueryBrowser
//
//  Created by Alfredo Kojima on Sat Mar 5 2005.
//  Copyright (c) 2005 MySQL AB. All rights reserved.
//


#import <Cocoa/Cocoa.h>

#import <MySQLToolsCommon/MSchemaDataSource.h>
#import <MySQLToolsCommon/MMySQLDispatcher.h>
#include <myx_public_interface.h>
#include <myx_qb_public_interface.h>

@class MTabView;
@class MSchemaDataSource;
@class MSchemaEditHelper;


// this does the role of a NSDocument
@interface MQueryBrowser : NSWindowController
{
  IBOutlet NSImageView *animbox;
  IBOutlet NSButton *backButton;
  IBOutlet NSMenu *backMenu;
  IBOutlet NSOutlineView *bookmarkList;
  IBOutlet NSOutlineView *catalogList;
  IBOutlet NSDrawer *drawer;
  IBOutlet NSButton *executeButton;
  IBOutlet NSMenu *executeMenu;
  IBOutlet NSOutlineView *historyList;
  IBOutlet NSButton *nextButton;
  IBOutlet NSMenu *nextMenu;
  IBOutlet NSOutlineView *parameterList;
  IBOutlet NSTextView *queryArea;
  IBOutlet NSButton *refreshButton;
  IBOutlet NSButton *stopButton;
  IBOutlet NSOutlineView *syntaxList;
  IBOutlet MTabView *tabView;
  IBOutlet NSTextField *statusText;
  
  IBOutlet MSchemaEditHelper *schemaHelper;
  
  NSString *_connectedInstance;
  
  NSImage *_idleImage;
  NSImage *_busyImage;
  
  NSImage *_catalogIcon;
  NSImage *_schemaIcon;
  NSImage *_tableIcon;
  NSImage *_columnIcon;
  NSImage *_keyIcon;
  
  NSMutableArray *_history;

  NSLock *_mysqlLock; // remove
  MYX_USER_CONNECTION *_info;
  MYSQL *_mysql;
  
  MMySQLDispatcher *_dispatcher;
  
  MYX_CATALOGS *_catalogs;
  MSchemaDataSource *_columnDS;
  
  MYX_SYN *_syntaxInfo;
  MYX_SQL_HIGHLIGHTING *_highlightInfo; 
  NSArray *_textColors;
  
  NSMutableArray *_queryViews;
}
- (IBAction)addQueryTab:(id)sender;
- (IBAction)changeMouse:(id)sender;
- (IBAction)commitTransaction:(id)sender;
- (IBAction)compareQuery:(id)sender;
- (IBAction)executeInNewTab:(id)sender;
- (IBAction)executeInSplitTab:(id)sender;
- (IBAction)executeQuery:(id)sender;
- (IBAction)explainQuery:(id)sender;
- (IBAction)goBack:(id)sender;
- (IBAction)goNext:(id)sender;
- (IBAction)reload:(id)sender;
- (IBAction)rollbackTransaction:(id)sender;
- (IBAction)startTransaction:(id)sender;
- (IBAction)stopQuery:(id)sender;
- (IBAction)toggleDrawer:(id)sender;

- (IBAction)refreshSchemata:(id)sender;
- (IBAction)setDefaultSchema:(id)sender;

// gains ownership of mysql and info
- (id)initWithConnection: (MYSQL*)mysql
                    info: (MYX_USER_CONNECTION*)info;


- (void)setTitleForPanel: (NSString*)name;

- (void)close;

- (void)unlockMySQL;
- (MYSQL*)mysql;
- (MYSQL*)mysqlLock;
- (MYX_USER_CONNECTION*)serverInfo;

- (MMySQLDispatcher*)dispatcher;

@end
