//
//  MQueryBrowser.m
//  MySQL QueryBrowser
//
//  Created by Alfredo Kojima on Sat Mar 5 2005.
//  Copyright (c) 2005 MySQL AB. All rights reserved.
//

#import "MQueryBrowserController.h"
#import "MQueryBrowser.h"

#import "MQResultSetView.h"
#import "MQResultSetTab.h"
#include <MySQLToolsCommon/mxUtils.h>

//#include "myx_qb_public_interface.h"

@interface MQueryBrowser(Private)
- (void)highlightQuery:(NSString*)query;
- (void)fetchSchemasFor:(MSchemaDataSource*)ds;
- (void)tablesArrived:(NSArray*)args result:(MYX_SCHEMA_TABLES*)tables;
- (void)requestTablesForSchema:(MSchemaDataSource*)sender item:(MSchemaItem*)item;
- (void)selectSchema:(NSString*)schema;
@end

@implementation MQueryBrowser(Private)

- (void)highlightQuery:(NSString*)query
{
  if (_syntaxInfo
	  && myx_highlight(_highlightInfo, [query UTF8String], 0) == 0
	  && _highlightInfo->words)
  {
	unsigned int i;
	
	for (i= 0; i < _highlightInfo->words_num; i++)
	{
	  NSRange range;
	  
	  if (_highlightInfo->words[i].word_begin == _highlightInfo->words[i].word_end)
		continue;
	  
	  range= NSMakeRange(_highlightInfo->words[i].word_begin,
						 _highlightInfo->words[i].word_end-_highlightInfo->words[i].word_begin+1);

	  [[queryArea textStorage] addAttribute:NSForegroundColorAttributeName
									  value:[_textColors objectAtIndex: _highlightInfo->words[i].word_type]
									  range:range];
	}
  }
}


- (void)fetchSchemasFor:(MSchemaDataSource*)ds
{
  MYX_CATALOGS *cats;
  
  cats= [_dispatcher performCallback:(void*(*)(MYSQL*))myx_get_catalogs
                       waitForWindow:[self window]
                             message:@"Retrieving catalog list..."];
  if (!cats)
  {
    NSLog(@"ERROR: Could not retrieve catalog list");
    return;
  }
  
  [ds resetTree];
  
  [ds updateWithCatalogs:cats];
  
  _catalogs= cats;
}


- (void)requestTablesForSchema:(MSchemaDataSource*)sender item:(MSchemaItem*)item
{
  MYX_SCHEMA *schema= [item schema];
 
  [_dispatcher performCallback:(void*(*)(MYSQL*,void*,void*))myx_get_schema_tables
                      argument:schema->catalog_name
                      argument:schema->schema_name
              finishedSelector:@selector(tablesArrived:result:)
                      argument:[[NSArray arrayWithObjects:sender,item,nil] retain]
                        target:self];
}


- (void)tablesArrived:(NSArray*)args result:(MYX_SCHEMA_TABLES*)tables
{
  MSchemaDataSource *ds= [args objectAtIndex:0];
  MSchemaItem *sitem= [args objectAtIndex:1];
  MYX_SCHEMA *schema= [sitem schema];

  [args autorelease];

  if (tables)
  {
    if (schema->schema_tables)
      myx_free_schema_tables(schema->schema_tables);
    schema->schema_tables= tables;
  }
  if (!tables)
  {
    NSLog(@"could not retrieve tables list");
    return;
  }
  
  [ds updateSchema:sitem 
		withTables:tables];
  
  [catalogList reloadData];
}

- (void)selectSchema:(NSString*)schema
{
  unsigned int i;
  
  g_free(_info->schema);
  _info->schema= schema == nil ? NULL : g_strdup([schema UTF8String]);
  if (schema)
	myx_use_schema(_mysql, [schema UTF8String]);  
  
  if (_info->schema)
  {
	_syntaxInfo= myx_refresh_dbinfo(_mysql, _syntaxInfo);
	if (_highlightInfo)
	  myx_free_sql_highlighting(_highlightInfo);
	_highlightInfo= myx_init_sql_parsing(_syntaxInfo);
  }
  if (schema)
  {
	[statusText setStringValue:[NSString stringWithFormat:@"Selected schema '%@'",schema]];
	for (i= 0; i < [_queryViews count]; i++)
	  [[_queryViews objectAtIndex:i] setDefaultSchema:schema];
  }
  else
	[statusText setStringValue:@"No schema selected."];  
}

@end

//------------------------------------------------------------------------------

@implementation MQueryBrowser

- (id)init
{ 
  self = [super initWithWindowNibName:@"QueryBrowser" owner:self];
  if (!self)
    return nil;
  
  [self window];
  if (![self isWindowLoaded])
    NSLog(@"ERROR: Could not load document window");
  
  _mysqlLock= [[NSLock alloc] init];
  
  _queryViews= [[NSMutableArray alloc] init];

  [[queryArea textStorage] setDelegate:self];
  [queryArea setFont:[NSFont fontWithName:@"Andale Mono" size:11.0]];

  _catalogIcon= [[NSImage imageNamed:@"16x16_Catalog.png"] retain];
  _schemaIcon= [[NSImage imageNamed:@"myx_schema_16x16.png"] retain];
  _tableIcon= [[NSImage imageNamed:@"myx_schema_table_16x16.png"] retain];
  _columnIcon= [[NSImage imageNamed:@"16x16_Field.png"] retain];
  _keyIcon= [[NSImage imageNamed:@"16x16_KeyColumn.png"] retain];

  _idleImage= [[NSImage imageNamed:@"sakila.png"] retain];
  _busyImage= [[NSImage imageNamed:@"dolphin_anim.gif"] retain];

  _textColors= [[NSArray arrayWithObjects:
	  [NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.0 alpha:1.0], // MYX_SYN_NORMAL
	  [NSColor colorWithDeviceRed:0.4 green:0.6 blue:0.9 alpha:1.0], // MYX_SYN_TABLE
	  [NSColor colorWithDeviceRed:0.2 green:0.4 blue:0.8 alpha:1.0], // MYX_SYN_COLUMN
	  [NSColor colorWithDeviceRed:0.1 green:0.5 blue:0.5 alpha:1.0], // MYX_SYN_COMMENT
	  [NSColor colorWithDeviceRed:0.2 green:0.5 blue:0.2 alpha:1.0], // MYX_SYN_STRING
	  [NSColor colorWithDeviceRed:0.5 green:0.5 blue:0.1 alpha:1.0], // MYX_SYN_SYMBOL
	  [NSColor colorWithDeviceRed:0.5 green:0.1 blue:0.3 alpha:1.0], // MYX_SYN_FUNCTION,
	nil] retain];

  _history= [[NSMutableArray alloc] init];
  
  [backButton setPeriodicDelay:1.0 interval:0.0];
  [backButton setPeriodicDelay:1.0 interval:0.0];
  [executeButton setPeriodicDelay:1.0 interval:0.0];
  
  return self;
}


- (id)initWithConnection: (MYSQL*)mysql
                    info: (MYX_USER_CONNECTION*)info
{
  self = [self init];
  
  if (self)
  {
    _mysql=mysql;
    _info= info;
    
    _dispatcher= [[MMySQLDispatcher alloc] initWithMySQL:mysql instance:info];
	_columnDS= [[MSchemaDataSource alloc] initWithRoot:MCatalogItemType 
												  leaf:MColumnItemType
												 icons:[NSDictionary dictionaryWithObjectsAndKeys:
												   [NSImage imageNamed:@"16x16_Catalog.png"], @"catalog",
												   [NSImage imageNamed:@"myx_schema_16x16.png"], @"schema",
												   [NSImage imageNamed:@"myx_schema_table_16x16.png"], @"table",
												   [NSImage imageNamed:@"16x16_Field.png"], @"column",
												   [NSImage imageNamed:@"16x16_KeyColumn.png"], @"key",
												   nil]];
	[self fetchSchemasFor:_columnDS];
	[_columnDS setTableFetcher:self selector:@selector(requestTablesForSchema:item:)];	
	
	[catalogList setDataSource:_columnDS];
	[catalogList reloadData];
	MXExpandOutline(catalogList, NO);
	  
    if (mysql->unix_socket)
      _connectedInstance= [[NSString alloc] initWithFormat:@"%s via socket", info->hostname];
    else
      _connectedInstance= [[NSString alloc] initWithFormat:@"%s:%i", info->hostname, info->port];
	
    [self selectSchema: info->schema?[NSString stringWithUTF8String:info->schema]:nil];
	
	[self addQueryTab:self];
  }
  return self;
}


- (void)dealloc
{
  [_queryViews release];
  [_dispatcher release];
  [_connectedInstance release];
  [_mysqlLock release];
  [_catalogIcon release];
  [_schemaIcon release];
  [_tableIcon release];
  [_columnIcon release];
  [_columnDS release];
  [_history release];
  
  if (_highlightInfo)
	myx_free_sql_highlighting(_highlightInfo);

  if (_syntaxInfo)
	myx_free_syn(_syntaxInfo);
  
  if (_catalogs)
    myx_free_catalogs(_catalogs);
  
  if (_mysql)
    myx_mysql_close(_mysql);
  if (_info)
    myx_free_user_connection_content(_info);
  
  [super dealloc];
}


- (IBAction)addQueryTab:(id)sender
{
  MQResultSetTab *page= [[MQResultSetTab alloc] initWithIdentifier:@"aa"];
  MQResultSetView *rsview;

  rsview= [[[MQResultSetView alloc] initWithConnectionTo:_info] autorelease];

  [rsview setDelegate:self];
  if (_mysql->db)
	[rsview setDefaultSchema:[NSString stringWithUTF8String:_mysql->db]];

  [page addResultSet:rsview];

  [page setLabel:@"Query"];
  [tabView addTabViewItem:page];
  [page release];
  
  [_queryViews addObject:rsview];
}


- (IBAction)changeMouse:(id)sender
{
}

- (IBAction)commitTransaction:(id)sender
{
}

- (IBAction)compareQuery:(id)sender
{
}

- (IBAction)executeInNewTab:(id)sender
{
}

- (IBAction)executeInSplitTab:(id)sender
{
}

- (IBAction)executeQuery:(id)sender
{
  NSString *query= [queryArea string];
  MQResultSetTab *page;
  MQResultSetView *rsview;

  page= (MQResultSetTab*)[tabView selectedPage];
  rsview= [page activeResultSet];

  [rsview performQuery:query];
}

- (IBAction)explainQuery:(id)sender
{
}

- (IBAction)goBack:(id)sender
{
  NSEvent *event= [NSApp currentEvent];
  if ([event type] == NSPeriodic)
  {
	NSPoint menuLocation = NSMakePoint(0, NSHeight([backButton frame]) - 8.0);
	menuLocation = [backButton convertPoint:menuLocation toView:nil];
	NSLog(@"%@", NSStringFromPoint(menuLocation));
	NSEvent *popEvent= [NSEvent mouseEventWithType: NSLeftMouseDown
					                      location: menuLocation
					                 modifierFlags: [event modifierFlags]
					                     timestamp: [event timestamp]
					                  windowNumber: [event windowNumber]
					                       context: [event context]
					                   eventNumber: 0
					                    clickCount: 1
					                      pressure: 1.0];
	
	[NSMenu popUpContextMenu:backMenu
				   withEvent:popEvent
					 forView:sender];
  }
  else
	NSLog(@"GO BACK %i", [[sender cell] isHighlighted]);  
}

- (IBAction)goNext:(id)sender
{
}

- (IBAction)reload:(id)sender
{
}

- (IBAction)rollbackTransaction:(id)sender
{
}

- (IBAction)startTransaction:(id)sender
{
}

- (IBAction)stopQuery:(id)sender
{
}

- (IBAction)toggleDrawer:(id)sender
{
  [drawer toggle:sender];
}


- (IBAction)refreshSchemata:(id)sender
{
  
}

- (IBAction)setDefaultSchema:(id)sender
{  
  NSString *schema;
  
  schema= [NSString stringWithUTF8String:[[catalogList itemAtRow:[catalogList selectedRow]] schema]->schema_name];
  [self selectSchema:schema];
}

- (MMySQLDispatcher*)dispatcher
{
  return _dispatcher;
}

- (void)setTitleForPanel: (NSString*)name
{
  if (name)
    [[self window] setTitle: [NSString stringWithFormat:@"%@  %@", 
      name, _connectedInstance]];
  else
    [[self window] setTitle: _connectedInstance];
}


- (void)close
{
//  if (!_closing)
//    [[self window] close];

  [[MQueryBrowserController sharedDocumentController] removeInstance:self];
}

- (void)unlockMySQL
{
  [_mysqlLock unlock];
}

- (MYSQL*)mysql
{
  return _mysql;
}

- (MYSQL*)mysqlLock
{
  [_mysqlLock lock];
  return _mysql;
}

- (MYX_USER_CONNECTION*)serverInfo
{
  return _info;
}

- (void)outlineView:(NSOutlineView *)outlineView 
    willDisplayCell:(id)cell 
     forTableColumn:(NSTableColumn *)tableColumn 
               item:(id)item
{
  if (outlineView == catalogList)
  {
	if ([item respondsToSelector:@selector(icon)])
	  [cell setImage:[item icon]];
	else
	  [cell setImage:nil];
  }
}


//-----------------------------------------------------------------------------

- (void)textStorageDidProcessEditing:(NSNotification *)aNotification
{
  id page= [[tabView selectedPage] representedObject];
  NSString *query= [queryArea string];
  
  [page setQuery:query];
  
  [self highlightQuery:query];
}

- (void)tabViewChangedPage:(MTabView*)sender
{
  id page= [[sender selectedPage] representedObject];
  
  if (page && [page query])
	[queryArea setString:[page query]];
  
  if (page && [page isBusy])
  {
	[animbox setImage:_busyImage];
	[stopButton setHidden:NO];
	[refreshButton setHidden:YES];
	[backButton setEnabled:NO];
	[nextButton setEnabled:NO];
	[executeButton setEnabled:NO];
	[queryArea setEditable:NO];	
  }
  else
  {
	[animbox setImage:_idleImage];
	[stopButton setHidden:YES];
	[refreshButton setHidden:NO];
	[backButton setEnabled:YES];
	[nextButton setEnabled:YES];
	[executeButton setEnabled:YES];  
	[queryArea setEditable:YES];	
  }
}

- (void)resultSetView:(MQResultSetView*)rsview willStartQuery:(NSString*)query
{
  [animbox setImage:_busyImage];
  [stopButton setHidden:NO];
  [refreshButton setHidden:YES];
  [backButton setEnabled:NO];
  [nextButton setEnabled:NO];
  [executeButton setEnabled:NO];
  [queryArea setEditable:NO];
}

- (void)resultSetViewFinishedQuery:(MQResultSetView*)rsview
{
  [animbox setImage:_idleImage];
  [stopButton setHidden:YES];
  [refreshButton setHidden:NO];
  [backButton setEnabled:YES];
  [nextButton setEnabled:YES];
  [executeButton setEnabled:YES];  
  [queryArea setEditable:YES];
}

- (int)numberOfItemsInMenu:(NSMenu *)menu
{
  return [menu numberOfItems];
}

- (BOOL)menu:(NSMenu *)menu
  updateItem:(NSMenuItem *)item
	 atIndex:(int)index
shouldCancel:(BOOL)shouldCancel
{
  int selRow= [catalogList selectedRow];
  int level= selRow < 0 ? -1 : [catalogList levelForRow:selRow];
  BOOL flag= NO;

  switch (index)
  {
	case 0: // Set Def. Schema
	  if (level > 0)
		flag= YES;
	  else
		flag= NO;
	  break;
	case 2: // Create Schema
	  if (level >= 0)
		flag= YES;
	  else
		flag= NO;
	  break;
	case 3: // Create Table
	  if (level >= 1)
		flag= YES;
	  else
		flag= NO;
	  break;	  
	case 4: // Edit
	  switch (level)
	  {
		case 0:
		  [item setTitle:@"Edit"];
		  flag= NO;
		  break;		  
		case 1:
		  [item setTitle:@"Edit Schema"];
		  flag= YES;
		  break;		  
		case 2:
		{
		  MSchemaItem *sitem= [catalogList itemAtRow:selRow];
		  [item setTitle:@"Edit Table"];
		  flag= YES;
		}
		  break;
		default:
		  [item setTitle:@"Edit"];
		  flag= NO;
		  break;
	  }
	  break;
	case 5: // Drop
	  switch (level)
	  {
		case 0:
		  [item setTitle:@"Drop"];
		  flag= NO;
		  break;
		case 1:
		  [item setTitle:@"Drop Schema"];
		  flag= YES;
		  break;
		case 2:
		{
		  MSchemaItem *sitem= [catalogList itemAtRow:selRow];
		  [item setTitle:@"Drop Table"];
		  flag= YES;
		  break;
		}
		default:
		  [item setTitle:@"Drop"];
		  flag= NO;
		  break;
	  }	  
	  break;
	case 7: // Refresh
	  flag= YES;
	  break;
  }
  
  [item setEnabled:flag];
  
  return YES;
}

@end
